#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

typedef NS_ENUM(NSInteger, MDMPMovieScalingMode) {
    MDMPMovieScalingModeNone,       // No scaling
    MDMPMovieScalingModeAspectFit,  // Uniform scale until one dimension fits
    MDMPMovieScalingModeAspectFill, // Uniform scale until the movie fills the visible bounds. One dimension may have clipped contents
    MDMPMovieScalingModeFill        // Non-uniform scale. Both render dimensions will exactly match the visible bounds
};

typedef NS_ENUM(NSInteger, MDMPMoviePlaybackState) {
    MDMPMoviePlaybackStateStopped,  // 播放停止
    MDMPMoviePlaybackStatePlaying,  // 开始播放
    MDMPMoviePlaybackStatePaused,   // 暂停播放
    MDMPMoviePlaybackStateInterrupted,  // 播放间断
    MDMPMoviePlaybackStateSeekingForward,   // 播放快进
    MDMPMoviePlaybackStateSeekingBackward   // 播放后退
};

typedef NS_OPTIONS(NSUInteger, MDMPMovieLoadState) {
    // 未知状态
    MDMPMovieLoadStateUnknown        = 0,
    MDMPMovieLoadStatePlayable       = 1 << 0,
    // 当shouldAutoPlay 为Yes时，将开始在这种状态
    MDMPMovieLoadStatePlaythroughOK  = 1 << 1, // Playback will be automatically started in this state when shouldAutoplay is YES
    // 播放后，自动设定为该方法
    MDMPMovieLoadStateStalled        = 1 << 2, // Playback will be automatically paused in this state, if started
};

//用于IJKMPMoviePlayerPlaybackDidFinishReasonUserInfoKey通知中，判断reason为一枚举
typedef NS_ENUM(NSInteger, MDMPMovieFinishReason) {
    MDMPMovieFinishReasonPlaybackEnded, // 完成原因：播放结束
    MDMPMovieFinishReasonPlaybackError, // 完成原因：播放出现错误
    MDMPMovieFinishReasonUserExited // 完成原因：出现用户行为退出
};

// -----------------------------------------------------------------------------
// Thumbnails
//获取在指定播放时间的视频缩略图，第一个参数是获取缩略图的时间点数组；第二个参数代表时间点精度，枚举类型
typedef NS_ENUM(NSInteger, MDMPMovieTimeOption) {
    MDMPMovieTimeOptionNearestKeyFrame,  // 时间点附近
    MDMPMovieTimeOptionExact     //准确时间
};

@protocol MDMediaPlayback;

#pragma mark MDMediaPlayback

@protocol MDMediaPlayback <NSObject>

- (void)prepareToPlay;
- (void)play;
- (void)pause;
- (void)stop;
- (BOOL)isPlaying;
- (void)shutdown;   // 这句要写在viewDidDisappear中，否则可能会造成vc不会被释放
- (void)setPauseInBackground:(BOOL)pause;

@property(nonatomic, readonly)  UIView *view;   // 用于显示视频播放的view，调用次view
@property(nonatomic)            NSTimeInterval currentPlaybackTime; // 当前播放的时间点
@property(nonatomic, readonly)  NSTimeInterval duration;    // 总时长
@property(nonatomic, readonly)  NSTimeInterval playableDuration;    // 可播放时长
@property(nonatomic, readonly)  NSInteger bufferingProgress;     // 通常从 0~100，缓冲区填充的字节百分比，如果因为网速不够快，停下来缓冲的时候，缓冲到100%时恢复播放，这个时候可以用来显示缓冲进度。播放时，缓冲也在进行，所以可以超过100

@property(nonatomic, readonly)  BOOL isPreparedToPlay;   // 准备播放
@property(nonatomic, readonly)  MDMPMoviePlaybackState playbackState;   // 播放状态枚举
@property(nonatomic, readonly)  MDMPMovieLoadState loadState;    // 加载状态枚举
@property(nonatomic, readonly) int isSeekBuffering;
@property(nonatomic, readonly) int isAudioSync;
@property(nonatomic, readonly) int isVideoSync;

@property(nonatomic, readonly) int64_t numberOfBytesTransferred;     // 传输字节数

@property(nonatomic, readonly) CGSize naturalSize;  // 视频原始显示View size
@property(nonatomic) MDMPMovieScalingMode scalingMode;  // 视频尺寸模式
@property(nonatomic) BOOL shouldAutoplay;   // 调用prepareToPlay准备好后自动播放

@property (nonatomic) BOOL allowsMediaAirPlay;  // 支持AirPlay 媒体
@property (nonatomic) BOOL isDanmakuMediaAirPlay;   // airplay is a feature of AVPlayer, not ijkplayer.
@property (nonatomic, readonly) BOOL airPlayMediaActive;    //AirPlay 是否活跃

@property (nonatomic) float playbackRate;   // 返回音频/视频的当前播放速度 0-1
@property (nonatomic) float playbackVolume;

- (UIImage *)thumbnailImageAtCurrentTime;    //获取当前时间的封面帧图片

#pragma mark Notifications

#ifdef __cplusplus
#define MD_EXTERN extern "C" __attribute__((visibility ("default")))
#else
#define MD_EXTERN extern __attribute__((visibility ("default")))
#endif

// -----------------------------------------------------------------------------
//  MPMediaPlayback.h

// Posted when the prepared state changes of an object conforming to the MPMediaPlayback protocol changes.
// This supersedes MPMoviePlayerContentPreloadDidFinishNotification.
// 播放状态的改变 代替MPMoviePlayerContentPreloadDidFinishNotification
MD_EXTERN NSString *const MDMPMediaPlaybackIsPreparedToPlayDidChangeNotification;

// -----------------------------------------------------------------------------
//  MPMoviePlayerController.h
//  Movie Player Notifications

// Posted when the scaling mode changes.
// 缩放比例的改变
MD_EXTERN NSString* const MDMPMoviePlayerScalingModeDidChangeNotification;

// Posted when movie playback ends or a user exits playback.
MD_EXTERN NSString* const MDMPMoviePlayerPlaybackDidFinishNotification;
//当电影播放结束或用户退出播放时调用。
MD_EXTERN NSString* const MDMPMoviePlayerPlaybackDidFinishReasonUserInfoKey; // NSNumber (MDMPMovieFinishReason)

// Posted when the playback state changes, either programatically or by the user.
MD_EXTERN NSString* const MDMPMoviePlayerPlaybackStateDidChangeNotification;

// Posted when the network load state changes.
MD_EXTERN NSString* const MDMPMoviePlayerLoadStateDidChangeNotification;

// Posted when the movie player begins or ends playing video via AirPlay.
MD_EXTERN NSString* const MDMPMoviePlayerIsAirPlayVideoActiveDidChangeNotification;

// -----------------------------------------------------------------------------
// Movie Property Notifications

// Calling -prepareToPlay on the movie player will begin determining movie properties asynchronously.
// These notifications are posted when the associated movie property becomes available.
// 在执行 prepareToPlay 时开始异步确定影片属性，当相关属性变为有效可用时调用该通知
MD_EXTERN NSString* const MDMPMovieNaturalSizeAvailableNotification;

// -----------------------------------------------------------------------------
//  Extend Notifications

// 视频 编译器打开通知
MD_EXTERN NSString *const MDMPMoviePlayerVideoDecoderOpenNotification;
// 视频 视频第一帧时通知
MD_EXTERN NSString *const MDMPMoviePlayerFirstVideoFrameRenderedNotification;
// 视频 音频第一段时通知
MD_EXTERN NSString *const MDMPMoviePlayerFirstAudioFrameRenderedNotification;
MD_EXTERN NSString *const MDMPMoviePlayerFirstAudioFrameDecodedNotification;
MD_EXTERN NSString *const MDMPMoviePlayerFirstVideoFrameDecodedNotification;
MD_EXTERN NSString *const MDMPMoviePlayerOpenInputNotification;
MD_EXTERN NSString *const MDMPMoviePlayerFindStreamInfoNotification;
MD_EXTERN NSString *const MDMPMoviePlayerComponentOpenNotification;

MD_EXTERN NSString *const MDMPMoviePlayerDidSeekCompleteNotification;
MD_EXTERN NSString *const MDMPMoviePlayerDidSeekCompleteTargetKey;
MD_EXTERN NSString *const MDMPMoviePlayerDidSeekCompleteErrorKey;
MD_EXTERN NSString *const MDMPMoviePlayerDidAccurateSeekCompleteCurPos;
MD_EXTERN NSString *const MDMPMoviePlayerAccurateSeekCompleteNotification;
MD_EXTERN NSString *const MDMPMoviePlayerSeekAudioStartNotification;
MD_EXTERN NSString *const MDMPMoviePlayerSeekVideoStartNotification;

MD_EXTERN NSString *const MDMPMoviePlayerApplicationDidBecomeActiveNotification;
MD_EXTERN NSString *const MDMPMoviePlayerApplicationDidEnterBackgroundNotification;

@end

#pragma mark MDMediaUrlOpenDelegate


typedef NS_ENUM(NSInteger, MDMediaEvent) {

    // Notify Events
    MDMediaEvent_WillHttpOpen         = 1,       // attr: url
    MDMediaEvent_DidHttpOpen          = 2,       // attr: url, error, http_code
    MDMediaEvent_WillHttpSeek         = 3,       // attr: url, offset
    MDMediaEvent_DidHttpSeek          = 4,       // attr: url, offset, error, http_code
    // Control Message
    MDMediaCtrl_WillTcpOpen           = 0x20001, // MediaUrlOpenData: no args
    MDMediaCtrl_DidTcpOpen            = 0x20002, // MediaUrlOpenData: error, family, ip, port, fd
    MDMediaCtrl_WillHttpOpen          = 0x20003, // MediaUrlOpenData: url, segmentIndex, retryCounter
    MDMediaCtrl_WillLiveOpen          = 0x20005, // MediaUrlOpenData: url, retryCounter
    MDMediaCtrl_WillConcatSegmentOpen = 0x20007, // MediaUrlOpenData: url, segmentIndex, retryCounter
};

#define MDMediaEventAttrKey_url            @"url"
#define MDMediaEventAttrKey_host           @"host"
#define MDMediaEventAttrKey_error          @"error"
#define MDMediaEventAttrKey_time_of_event  @"time_of_event"
#define MDMediaEventAttrKey_http_code      @"http_code"
#define MDMediaEventAttrKey_offset         @"offset"
#define MDMediaEventAttrKey_file_size      @"file_size"

// event of MDMediaUrlOpenEvent_xxx
@interface MDMediaUrlOpenData: NSObject

- (id)initWithUrl:(NSString *)url
            event:(MDMediaEvent)event
     segmentIndex:(int)segmentIndex
     retryCounter:(int)retryCounter;

@property(nonatomic, readonly) MDMediaEvent event;
@property(nonatomic, readonly) int segmentIndex;
@property(nonatomic, readonly) int retryCounter;     // 重试次数

@property(nonatomic, retain) NSString *url;
@property(nonatomic, assign) int fd;
@property(nonatomic, strong) NSString *msg;
@property(nonatomic) int error; // set a negative value to indicate an error has occured.
@property(nonatomic, getter=isHandled)    BOOL handled;     // auto set to YES if url changed
@property(nonatomic, getter=isUrlChanged) BOOL urlChanged;  // auto set to YES by url changed

@end

@protocol MDMediaUrlOpenDelegate <NSObject>

- (void)willOpenUrl:(MDMediaUrlOpenData*) urlOpenData;

@end

@protocol MDMediaNativeInvokeDelegate <NSObject>

- (int)invoke:(MDMediaEvent)event attributes:(NSDictionary *)attributes;

@end
