//
//  MDPlayerLiveView.h
//  MDPlayerManager
//
//  Created by mac-zwj on 2022/4/1.
//

#import <UIKit/UIKit.h>
#import <MDPlayerManager/MDPlayerDelegate.h>

NS_ASSUME_NONNULL_BEGIN

@interface MDPlayerLiveView : UIView


#pragma mark - Common

/**
 * @brief 设置Adapter
 * @note 播放视频以及后续的所有功能操作都基于设置完Adapter才有效
 */
@property (nonatomic, strong) id<MDPlayDelegate> adapter;

/**
 * @brief 播放器是否正在播放中。YES:正在播放中；NO:未在播放。
 * @note 暂停不属于正在播放中，会返回NO
 */
@property (nonatomic, assign, readonly) BOOL isPlaying;

/**
 * @brief 播放器是否正在暂停中。YES:正在暂停中；NO:没有在暂停。
 */
@property (nonatomic, assign, readonly) BOOL isPaused;

/**
 * @brief 是否显示播放器状态信息的调试浮层。默认NO
 */
@property (nonatomic, assign) BOOL isShowDebugView;

/**
 * @brief APP进入后台时是否暂停播放
 */
@property (nonatomic, assign) BOOL pauseInBackground;

/**
 * 直播播放器错误通知，播放器出现错误时，会回调该通知
 * @param code      错误码
 * @param msg       错误信息
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onError)(MDPlayerCode code, NSString* message, NSDictionary* extraInfo);

/**
 * 直播播放器警告通知
 * @param code      警告码
 * @param msg       警告信息
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onWarning)(MDPlayerCode code, NSString* message, NSDictionary* extraInfo);

/**
 * @brief 截图回调
 * @note  调用 [snapshot] 截图之后，会收到这个回调通知
 * @param image  已截取的视频画面
 */
@property (nonatomic, copy) void(^onSnapshotComplete)(UIImage* image);

/**
 * @brief 播放器进入后台的回调
 */
@property (nonatomic, copy) void(^onPlayerEnterBackground)(void);

/**
 * @brief 播放器进入前台的回调
 */
@property (nonatomic, copy) void(^onPlayerBecomeActive)(void);

/**
 * @brief 获取MDLiveView对象
 */
+ (instancetype)liveView;

/**
 * @brief 开始播放
 * @param url 播放地址
 */
- (MDPlayerCode)startPlay:(NSString*)url;

/**
 * @brief 停止播放
 */
- (MDPlayerCode)stopPlay;

/**
 * @brief 暂停播放
 */
- (MDPlayerCode)pausePlay;

/**
 * @brief 恢复播放
 */
- (MDPlayerCode)resumePlay;

/**
 * @brief 销毁
 */
- (MDPlayerCode)destory;

/**
 * @brief 设置画面填充模式。
 * @note MDRealTimeStreamingPlayerAdapter 只有 MDPlayerScaleModeAspectFit 和 MDPlayerScaleModeAspectFill 两种模式可以设置，设置其他值都视为 MDPlayerScaleModeAspectFit。
 */
- (MDPlayerCode)setScaleMode:(MDPlayerScaleMode)scaleMode;

/**
 * @brief 获取画面填充模式。
 */
- (MDPlayerScaleMode)getScaleMode;

/**
 * @brief 设置播放器音量。取值范围0 - 100。默认100
 */
- (MDPlayerCode)setVolume:(NSUInteger)volume;

/**
 * @brief 获取音量值
 */
- (NSUInteger)getVolume;

/**
 * @brief 截取播放过程中的视频画面。通过onSnapshotComplete回调来获取图片。
 */
- (MDPlayerCode)snapshot;


#pragma mark - MDMediaPlayerAdapter

/**
 * @brief 播放速度。默认1.0。
 */
@property (nonatomic, assign) float playbackRate;

/**
 * @brief 当前播放时间。设置播放进度时调用。
 */
@property (nonatomic, assign) float currentPlaybackTime;

/**
 * @brief 视频总时长。
 */
@property (nonatomic, readonly) NSTimeInterval duration;

/**
 * @brief 可播放时长。
 */
@property (nonatomic, readonly) NSTimeInterval playableDuration;

/**
 * @brief 是否准备好播放。
 */
@property (nonatomic, readonly) BOOL isPreparedToPlay;

/**
 * @brief 播放状态
 */
@property (nonatomic, readonly) MDPlaybackState playbackState;

/**
 * @brief 视频源的尺寸
 */
@property (nonatomic, readonly) CGSize naturalSize;

/**
 * @bried 播放开始的回调
 */
@property (nonatomic, copy) void(^onPlaybackIsPrepared)(void);

/**
 * @brief 播放结束的回调
 */
@property (nonatomic, copy) void(^onPlaybackDidFinished)(void);


#pragma mark - MDRealTimeStreamingPlayer

/**
 * @brief 音频是否正在暂停中。YES:正在暂停中；NO:没有在暂停。
 */
@property (nonatomic, assign, readonly) BOOL isPausedAudio;

/**
 * @brief 视频是否正在暂停中。YES:正在暂停中；NO:没有在暂停。
 */
@property (nonatomic, assign, readonly) BOOL isPausedVideo;

/**
 * @brief 播放音量大小提示频率。开启后可以在 onPlayoutVolumeUpdate 回调中获取到 SDK 对音量大小值的评估。决定了onPlayoutVolumeUpdate回调的触发间隔， 单位为ms， 最小间隔为100ms， 建议设置为300ms， 如果小于等于0则会关闭回调。 【默认值】：0，不开启
 */
@property (nonatomic, assign) NSInteger volumeEvaluationMs;

/**
 * @brief 播放器分辨率变化通知
 * @param width     视频宽
 * @param height    视频高
 */
@property (nonatomic, copy) void(^onVideoResolutionChanged)(NSInteger width, NSInteger height);
/**
 * @brief 已经成功连接到服务器
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onConnected)(NSDictionary* extraInfo);
/**
 * @brief 视频播放事件
 * @param firstPlay 第一次播放标志
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onVideoPlaying)(BOOL firstPlay, NSDictionary* extraInfo);
/**
 * @brief 音频播放事件
 * @param firstPlay 第一次播放标志
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onAudioPlaying)(BOOL firstPlay, NSDictionary* extraInfo);
/**
 * @brief 视频加载事件
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onVideoLoading)(NSDictionary* extraInfo);
/**
 * @brief 音频加载事件
 * @param extraInfo 扩展信息
 */
@property (nonatomic, copy) void(^onAudioLoading)(NSDictionary* extraInfo);
/**
 * @brief 播放器音量大小回调
 * @param volume 音量大小
 * @note  设置 volumeEvaluationMs 开启播放音量大小提示之后，会收到这个回调通知。
 */
@property (nonatomic, copy) void(^onPlayoutVolumeUpdate)(NSInteger volume);
/**
 * @brief 播放器统计数据回调
 * @param statistics 播放器统计数据
 */
@property (nonatomic, copy) void(^onStatisticsUpdate)(id statistics);
/**
 * @brief 自定义视频渲染回调
 * @param videoFrame 视频帧数据
 * @note  需要您调用 [enableObserveVideoFrame] 开启回调开关
 */
@property (nonatomic, copy) void(^onRenderVideoFrame)(id videoFrame);
/**
 * @brief 收到 SEI 消息的回调，发送端通过 `sendSeiMessage` 来发送 SEI 消息。
 * @note  调用 `enableReceiveSeiMessage` 开启接收 SEI 消息之后，会收到这个回调通知。
 * @param payloadType    回调数据的SEI payloadType
 * @param data     数据
 */
@property (nonatomic, copy) void(^onReceiveSeiMessage)(int payloadType, NSData* data);

/**
 * @brief 暂停音频流。
 */
- (MDPlayerCode)pauseAudio;

/**
 * @brief 恢复音频流。
 */
- (MDPlayerCode)resumeAudio;

/**
 * @brief 暂停视频流。
 */
- (MDPlayerCode)pauseVideo;

/**
 * @brief 恢复视频流。
 */
- (MDPlayerCode)resumeVideo;

/**
 * @brief 设置视频画面顺时针旋转的角度。默认不旋转
 */
- (MDPlayerCode)setRenderRotation:(MDPlayerRenderRotation)rotation;

/**
 * @brief 获取视频画面顺时针旋转的角度。
 */
- (MDPlayerRenderRotation)getRenderRotation;

/**
 * @brief 设置播放器缓存自动调整的最小和最大时间 ( 单位：秒 )。
 * @param minTime 缓存自动调整的最小时间，取值需要大于0。【默认值】：1。
 * @param maxTime 缓存自动调整的最大时间，取值需要大于0。【默认值】：5。
 * @note minTime 和 maxTime 需要大于0，修改时播放器不能处于播放状态。这里有3种建议模式：
 * 1.极速模式： 参数分别传1、1； 优点是平均延迟较低2s-3s， 缺点是卡顿率偏高； 在延迟控制上有优势， 适用于对延迟大小比较敏感的场景。
 * 流畅模式： 参数分别传5、5； 优点是卡顿率最低， 缺点是延迟率较高>=5s； 对于超大码率的游戏直播（例如绝地求生）非常适合。
 * 自动模式： 参数分别传1、5； 卡顿率和延时率根据网络自适应； 适用于混合场景。
 */
- (MDPlayerCode)setCacheParams:(CGFloat)minTime maxTime:(CGFloat)maxTime;

/**
 * @brief 开启/关闭对视频帧的监听回调。SDK 在您开启次此开关后将不再渲染视频画面，您可以通过 onRenderVideoFrame 获得视频帧，并执行自定义的渲染逻辑。
 * @param enable      是否开启自定义渲染。【默认值】：NO
 * @param pixelFormat 自定义渲染回调的视频像素格式。
 * @param bufferType  自定义渲染回调的视频数据格式。
 */
- (MDPlayerCode)enableObserveVideoFrame:(BOOL)enable pixelFormat:(MDPlayerPixelFormat)pixelFormat bufferType:(MDPlayerBufferType)bufferType;

/**
 * @brief 开启/关闭接收 SEI 消息。
 * @param enable YES:开启接收SEI消息; NO:关闭接收SEI消息。【默认值】: NO
 * @param payloadType 指定接收SEI消息的payloadType，支持5、242，请与发送端的payloadType保持一致。
 */
- (MDPlayerCode)enableReceiveSeiMessage:(BOOL)enable payloadType:(int)payloadType;

@end

NS_ASSUME_NONNULL_END
